import type { ColorValue, StyleProp, TextStyle } from 'react-native'
import type { ThemedToken, ThemeInput, ThemeRegistration } from 'shiki'
import type { SyntaxHighlighterProps } from 'src/types/shiki'
import React, { useMemo } from 'react'
import { Platform, ScrollView, StyleSheet, Text, View } from 'react-native'
import { useSyntaxHighlighter } from '../../hooks/useSyntaxHighlighter'
import { getRNStylesFromShikiStyle } from '../../utils/style-transformer'

const monospaceFont = Platform.select({
  ios: 'Menlo',
  android: 'monospace',
  default: 'monospace',
})

interface TokenStyle extends Omit<TextStyle, 'color' | 'fontWeight'> {
  color?: ColorValue
  fontStyle?: 'normal' | 'italic'
  fontWeight?: FontWeight
  fontFamily?: string
}

type FontWeight = 'normal' | 'bold' | '100' | '200' | '300' | '400' | '500' | '600' | '700' | '800' | '900'

function getThemeRegistration(theme: ThemeInput): ThemeRegistration {
  if ('default' in theme) {
    return theme.default
  }
  return theme as ThemeRegistration
}

const baseStyles = StyleSheet.create({
  scrollView: {
    flex: 1,
    minHeight: 20,
    backgroundColor: '#1e1e1e',
  },
  lineContainer: {
    flexDirection: 'row',
    flexWrap: 'wrap',
    minWidth: '100%',
    paddingHorizontal: 16,
    paddingVertical: 4,
  },
  token: {
    fontFamily: monospaceFont,
    fontSize: 14,
    color: '#d4d4d4',
  },
  container: {
    paddingVertical: 8,
  },
  line: {
    flexDirection: 'row',
    flexWrap: 'wrap',
    width: '100%',
  },
})

const SyntaxHighlighter: React.FC<SyntaxHighlighterProps> = ({ text, language, languageId, theme, fontSize = 14 }) => {
  const stylesheet = useMemo(() => {
    const themeRegistration = getThemeRegistration(theme)
    const styles = getRNStylesFromShikiStyle(themeRegistration)

    return StyleSheet.create({
      ...styles,
      editor: {
        backgroundColor: themeRegistration.colors?.['editor.background'] || '#1e1e1e',
      },
      token: {
        ...baseStyles.token,
        fontSize,
        color: themeRegistration.colors?.['editor.foreground'] || '#d4d4d4',
      },
    })
  }, [theme, fontSize])

  const { tokens } = useSyntaxHighlighter({
    text,
    language,
    languageId,
    theme,
  })

  const renderToken = (token: ThemedToken, index: number) => {
    const styles: StyleProp<TokenStyle>[] = [stylesheet.token]

    // Always apply the token color if it exists
    if (token.color) {
      styles.push({ color: token.color })
    }

    // Add font styles
    if (token.fontStyle) {
      styles.push({
        // fontStyle: token.fontStyle.includes('italic') ? 'italic' : 'normal',
        // fontWeight: token.fontStyle.includes('bold') ? 'bold' : 'normal',
        fontStyle: token.fontStyle as unknown as 'normal' | 'italic',
        fontWeight: (token.fontStyle as unknown as string).includes('bold') ? 'bold' : 'normal',
      })
    }

    return (
      <Text key={index} style={styles}>
        {token.content.replace(/ /g, '\u00A0')}
      </Text>
    )
  }

  const renderLine = (line: ThemedToken[], lineIndex: number) => (
    <View key={lineIndex} style={baseStyles.line}>
      {line.map((token, tokenIndex) => renderToken(token, tokenIndex))}
      <Text>{'\n'}</Text>
    </View>
  )

  return (
    <ScrollView horizontal showsHorizontalScrollIndicator={Platform.OS !== 'web'} style={[baseStyles.scrollView, stylesheet.editor]} contentContainerStyle={baseStyles.container}>
      <View>{tokens.map((line, index) => renderLine(line, index))}</View>
    </ScrollView>
  )
}

export default SyntaxHighlighter
